package com.lantone.qc.kernel;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.gson.JsonObject;
import com.lantone.qc.pub.jdbc.MysqlJdbc;
import com.lantone.qc.pub.util.FastJsonUtils;
import com.lantone.qc.pub.util.PropertiesUtil;
import com.lantone.qc.trans.changx.util.CxXmlUtil;
import com.lantone.qc.trans.taizhou.util.TzXmlUtil;
import com.lantone.qc.trans.util.http.HttpApi;
import org.apache.commons.lang3.StringUtils;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.sql.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.zip.GZIPInputStream;

/**
 * @ClassName : TaizDataImportApiTest
 * @Description :
 * @Author : 楼辉荣
 * @Date: 2020-03-31 19:34
 */
public class TaizDataImportApiTest {
    private static String hospitalId = "1";
    private static String cid = "changx";

    private Map<String,String> caseMap = new HashMap<>();

    private List<String> deletePatient = new ArrayList<>();

    public static void main(String[] args) {
        TaizDataImportApiTest test = new TaizDataImportApiTest();
        //模块映射
        test.loadModelHospital();
        //长兴患者信息
        if (cid.equals("changx")) {
            test.insertPatient(test.loadOraclePatients("BR_INPATIENTINFO_CX"));
        }
        //台州患者信息
        if (cid.equals("taizhou")) {
            test.insertPatient(test.loadOraclePatients("BR_INPATIENTINFO_TZ20200330"));
        }
        //病人流水号获取    病历号--->id
        Map<String, String> patientMap = test.loadPatients(hospitalId);
        //model-->id
        Map<String, String> modelMap = test.loadModel();
        test.insertPatientText(patientMap, modelMap);
    }

    private void insertPatient(List<Patient> patients) {
        List<Map<String, Object>> list = new ArrayList<>();
        for (Patient patient : patients) {
            Map<String, Object> patientMap = new HashMap<>();
            patientMap.put("case_number", patient.getBrzyid());
            patientMap.put("hospital_id", hospitalId);
            list.add(patientMap);
        }
        initMysqlJdbc().insert(list, "qc_cases_number_copy", new String[]{"hospital_id", "case_number"});
    }

    private void insertPatientText(Map<String, String> patientMap, Map<String, String> modelMap) {
        List<Map<String, Object>> list = new ArrayList<>();

        for (Map.Entry<String, String> entry : patientMap.entrySet()) {
            List<PatientText> patientTexts = new ArrayList<>();
            //病人的病案首页内容
            Map<String, String> homePageMap = DataTest.loadHomePage(entry.getKey());
            if (homePageMap == null || homePageMap.isEmpty()) { //没病案首页直接不导入
                deletePatient.add(entry.getKey()); //没病案首页的患者也要删除
                continue;
            }
            Map<String, Object> firstPageTextMap = new HashMap<>();
            firstPageTextMap.put("case_number", entry.getKey());
            firstPageTextMap.put("case_number_id", patientMap.get(entry.getKey()));
            firstPageTextMap.put("hospital_id", hospitalId);
            firstPageTextMap.put("mode_id", 6);
            firstPageTextMap.put("origin_mode", "病案首页");
            firstPageTextMap.put("origin_text", FastJsonUtils.getBeanToJson(homePageMap));
            String pagetext = "";
            for (Map.Entry<String, String> e : homePageMap.entrySet()) {
                pagetext = pagetext + "【" + e.getKey() + "】：" + e.getValue().toString() + "\n";
            }
            firstPageTextMap.put("text", pagetext);

            if ("changx".equals(cid)) {
                patientTexts = loadChangxOraclePatientTexts(entry.getKey());
            }
            if ("taizhou".equals(cid)) {
                patientTexts = loadTaizhouOraclePatientTexts(entry.getKey());
            }
            if (!wholePatientRecord(patientTexts, modelMap) || patientTexts.size() == 0) {
                continue;
            }
            //数据完整后才放入病案首页
            list.add(firstPageTextMap);
            for (PatientText patientText : patientTexts) {
//System.out.println(patientText.getBrzyid() + "  " + patientText.getBljlmc());
                Map<String, Object> patientTextMap = new HashMap<>();
                patientTextMap.put("case_number", patientText.getBrzyid());
                patientTextMap.put("case_number_id", patientMap.get(patientText.getBrzyid()));
                patientTextMap.put("hospital_id", hospitalId);
//System.out.println("******" + patientText.getBljlmc() + "**" + caseMap.get(patientText.getBljlmc()));
                patientTextMap.put("mode_id", modelMap.get(caseMap.get(patientText.getBljlmc())));
                patientTextMap.put("origin_mode", patientText.getBljlmc());
                if (patientTextMap.get("mode_id") == null) {
                    continue;
                }
                patientTextMap.put("origin_text", patientText.getBljlnr());

                String text = "";
                Map<String, String> textMap = new HashMap<>();
                if ("changx".equals(cid)) {
                    textMap = CxXmlUtil.firstLevelNodeValue(patientText.getBljlnr(), caseMap.get(patientText.getBljlmc()));
                }
                if ("taizhou".equals(cid)) {
                    textMap = TzXmlUtil.getXmlToMapForTZ(patientText.getBljlnr());
                }
                for (Map.Entry<String, String> e : textMap.entrySet()) {
                    text = text + "【" + e.getKey() + "】：" + e.getValue() + "\n";
                }
                patientTextMap.put("text", text);
                list.add(patientTextMap);
            }
        }
        initMysqlJdbc().insert(list, "qc_model_mapping_copy", new String[]{"case_number", "case_number_id", "origin_text", "mode_id", "text", "origin_mode"});

        deletePatientInfos(deletePatient);
    }

    /**
     * 验证数据完整性
     * @param patientTexts
     * @param modelMap
     * @return
     */
    private boolean wholePatientRecord(List<PatientText> patientTexts, Map<String, String> modelMap) {
        Map<String, String> m = new HashMap<>();
        for (PatientText patientText : patientTexts) {
            String modelId = modelMap.get(caseMap.get(patientText.getBljlmc()));
            if (StringUtils.isEmpty(modelId)) {
                continue;
            }
            if (modelId.equals("1") && patientText.getBljlnr().indexOf("<") != 0) {
                deletePatient.add(patientTexts.get(0).getBrzyid());
                return false;
            }
            m.put(modelId, modelId);
        }
        if (m.get("1") != null && m.get("2") != null && m.get("4") != null && m.get("5") != null) {
            return true;
        } else {
            deletePatient.add(patientTexts.get(0).getBrzyid());
        }
        return false;
    }

    private List<Patient> loadOraclePatients(String tableName) {
        MysqlJdbc mysqlJdbc = initOracleJdbc();
        Connection conn = mysqlJdbc.connect();
        Statement st = null;
        ResultSet rs = null;
        String r1, r2;
        List<Patient> patients = new ArrayList<>();
        try {
            st = conn.createStatement();
            String sql = "select BRZYID from " + tableName;
            rs = st.executeQuery(sql);
            while (rs.next()) {
                r1 = rs.getString(1);
                Patient patient = new Patient();
                patient.setBrzyid(r1);
                patients.add(patient);
            }
        } catch (SQLException sqle) {
            sqle.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            mysqlJdbc.close(rs, st, conn);
        }
        return patients;
    }

    private List<PatientText> loadTaizhouOraclePatientTexts(String brzyid) {
        MysqlJdbc mysqlJdbc = initOracleJdbc();
        Connection conn = mysqlJdbc.connect();
        Statement st = null;
        ResultSet rs = null;
        String r1, r2, r3;
        List<PatientText> patientTexts = new ArrayList<>();
        try {
            st = conn.createStatement();
            String sql = "select brzyid,bljlmc,bljlnr from mr_medicalrecords_TZ20200330 a " +
                    "left join mr_mrcontent_TZ20200330 b on a.bljlid=b.bljlid " +
                    "where bljlnr is not null and brzyid = '" + brzyid + "' " +
                    "order by brzyid";
            rs = st.executeQuery(sql);
            while (rs.next()) {
                r1 = rs.getString(1);
                r2 = rs.getString(2);
                r3 = rs.getString(3);
                PatientText patientText = new PatientText();
                patientText.setBrzyid(r1);
                patientText.setBljlmc(r2);
                patientText.setBljlnr(r3);
                patientTexts.add(patientText);
            }
        } catch (SQLException sqle) {
            sqle.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            mysqlJdbc.close(rs, st, conn);
        }
        return patientTexts;
    }
    private List<PatientText> loadChangxOraclePatientTexts(String brzyid) {
        MysqlJdbc mysqlJdbc = initOracleJdbc();
        Connection conn = mysqlJdbc.connect();
        Statement st = null;
        ResultSet rs = null;
        String r1, r2;
        List<PatientText> patientTexts = new ArrayList<>();
        try {
            st = conn.createStatement();
            String sql = "select brzyid,bllbid,bljlnr from MR_MEDICALRECORDS_CX a " +
                    "left join MR_MRCONTENT_CX b on a.bljlid=b.bljlid " +
                    "where bljlnr is not null and brzyid = '" + brzyid + "' " +
                    "order by brzyid";
            rs = st.executeQuery(sql);
            while (rs.next()) {
                r1 = rs.getString(1);
                r2 = rs.getString(2);
                java.sql.Blob blob = rs.getBlob(3);

                InputStream in = blob.getBinaryStream();

                GZIPInputStream ginzip = new GZIPInputStream(in);

                ByteArrayOutputStream out = new ByteArrayOutputStream();
                byte[] buffer = new byte[1024];
                int offset = -1;
                while ((offset = ginzip.read(buffer)) != -1) {
                    out.write(buffer, 0, offset);
                }
                String decompressed = out.toString();
                in.close();
                out.close();
                ginzip.close();

                PatientText patientText = new PatientText();
                patientText.setBrzyid(r1);
                patientText.setBljlmc(r2);
                patientText.setBljlnr(decompressed);
                patientTexts.add(patientText);

//                String[] decos = decompressed.split("<\\?");
//                for (String s : decos) {
//                    if (StringUtils.isEmpty(s)) continue;
//                    PatientText patientText = new PatientText();
//                    patientText.setBrzyid(r1);
//                    patientText.setBljlmc(r2);
//
//                    s = "<?" + s;
//                    s = CxXmlUtil.cutXml(s);
//                    patientText.setBljlnr(s);
//                    patientTexts.add(patientText);
//                }
            }
        } catch (SQLException sqle) {
            sqle.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            mysqlJdbc.close(rs, st, conn);
        }
        return patientTexts;
    }

    private Map<String, String> loadPatients(String hospitalId) {
        MysqlJdbc mysqlJdbc = initMysqlJdbc();
        Connection conn = mysqlJdbc.connect();
        Statement st = null;
        ResultSet rs = null;
        String r1, r2;
        Map<String, String> map = new HashMap<>();
        try {
            st = conn.createStatement();
            String sql = "select id, case_number from qc_cases_number where hospital_id = " + hospitalId;
            rs = st.executeQuery(sql);
            while (rs.next()) {
                r1 = rs.getString(1);
                r2 = rs.getString(2);
                map.put(r2, r1);
            }
        } catch (SQLException sqle) {
            sqle.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            mysqlJdbc.close(rs, st, conn);
        }
        return map;
    }

    private Map<String, String> loadModel() {
        MysqlJdbc mysqlJdbc = initMysqlJdbc();
        Connection conn = mysqlJdbc.connect();
        Statement st = null;
        ResultSet rs = null;
        String r1, r2;
        Map<String, String> map = new HashMap<>();
        try {
            st = conn.createStatement();
            String sql = "select id, name from qc_mode";
            rs = st.executeQuery(sql);
            while (rs.next()) {
                r1 = rs.getString(1);
                r2 = rs.getString(2);
                map.put(r2, r1);
            }
        } catch (SQLException sqle) {
            sqle.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            mysqlJdbc.close(rs, st, conn);
        }
        return map;
    }

    private void loadModelHospital() {
        MysqlJdbc mysqlJdbc = initMysqlJdbc();
        Connection conn = mysqlJdbc.connect();
        Statement st = null;
        ResultSet rs = null;
        String r1, r2;
        try {
            st = conn.createStatement();
            String sql = "select hospital_model_name, stand_model_name from qc_model_hospital where stand_model_name is not null and hospital_id = " + hospitalId;
            rs = st.executeQuery(sql);
            while (rs.next()) {
                r1 = rs.getString(1);
                r2 = rs.getString(2);
                caseMap.put(r1, r2);
            }
        } catch (SQLException sqle) {
            sqle.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            mysqlJdbc.close(rs, st, conn);
        }
    }

    private void deletePatientInfos(List<String> caseNumbers) {
        MysqlJdbc mysqlJdbc = initMysqlJdbc();
        Connection conn = mysqlJdbc.connect();
        Statement st = null;
        ResultSet rs = null;
        String r1, r2;
        try {
            st = conn.createStatement();
            for (String caseNumber : caseNumbers) {
                String sql = "delete from qc_cases_number where case_number = '" + caseNumber + "'";
                st.execute(sql);
            }
        } catch (SQLException sqle) {
            sqle.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            mysqlJdbc.close(rs, st, conn);
        }
    }

    private MysqlJdbc initMysqlJdbc() {
        PropertiesUtil propertiesUtil = new PropertiesUtil("kernel.properties");
        MysqlJdbc mysqlJdbc = new MysqlJdbc(propertiesUtil.getProperty("mysql.test.user"),
                propertiesUtil.getProperty("mysql.test.password"),
                propertiesUtil.getProperty("mysql.test.url"));
        return mysqlJdbc;
    }

    private MysqlJdbc initOracleJdbc() {
        PropertiesUtil propertiesUtil = new PropertiesUtil("kernel.properties");
        MysqlJdbc mysqlJdbc = new MysqlJdbc(propertiesUtil.getProperty("oracle.test.user"),
                propertiesUtil.getProperty("oracle.test.password"),
                propertiesUtil.getProperty("oracle.test.url"), propertiesUtil.getProperty("oracle.test.driver"));
        return mysqlJdbc;
    }
}