package com.lantone.qc.trans.changx;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import com.lantone.qc.pub.model.doc.operation.OperationDiscussionDoc;
import com.lantone.qc.pub.model.doc.operation.OperationDoc;
import com.lantone.qc.pub.model.doc.operation.OperationRecordDoc;
import com.lantone.qc.pub.model.doc.operation.PreoperativeDiscussionDoc;
import com.lantone.qc.pub.model.vo.MedrecVo;
import com.lantone.qc.pub.util.ListUtil;
import com.lantone.qc.pub.util.StringUtil;
import com.lantone.qc.trans.ModelDocTrans;
import com.lantone.qc.trans.changx.util.CxXmlUtil;
import com.lantone.qc.trans.comsis.ModelDocGenerate;
import com.lantone.qc.trans.comsis.OrdinaryAssistant;
import com.lantone.qc.trans.comsis.Preproc;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @Description: 手术文档生成
 * @author: rengb
 * @time: 2020/3/20 17:11
 */
public class ChangxOperationDocTrans extends ModelDocTrans {

    @Override
    public List<OperationDoc> extract(MedrecVo medrecVo) {
        List<OperationDoc> retList = Lists.newArrayList();

        Map<String, List<String>> contentMap = (Map) medrecVo.getContent().get("content");
        if (contentMap == null) {
            return retList;
        }

        Map<String, OperationDiscussionDoc> operationDiscussionDocMap = getOperationDiscussionDocMap(contentMap.get("术后首次病程及谈话记录"));
        Map<String, OperationRecordDoc> operationRecordDocMap = getOperationRecordDocMap(contentMap.get("手术记录"));
        Map<String, PreoperativeDiscussionDoc> preoperativeDiscussionDocMap = getPreoperativeDiscussionDocMap(contentMap.get("术前讨论、术前小结"));

        Set<String> operationNameSet = Sets.newHashSet();
        operationNameSet.addAll(operationDiscussionDocMap.keySet());
        operationNameSet.addAll(operationRecordDocMap.keySet());
        operationNameSet.addAll(preoperativeDiscussionDocMap.keySet());

        operationNameSet.forEach(operationName -> {
            OperationDoc operationDoc = new OperationDoc();
            operationDoc.setOperationName(operationName);
            operationDoc.setOperationDiscussionDoc(operationDiscussionDocMap.get(operationName));
            operationDoc.setOperationRecordDoc(operationRecordDocMap.get(operationName));
            operationDoc.setPreoperativeDiscussionDoc(preoperativeDiscussionDocMap.get(operationName));
            retList.add(operationDoc);
        });

        return retList;
    }


    /*******************************************术后首次病程及谈话记录***************************************************/
    private Map<String, OperationDiscussionDoc> getOperationDiscussionDocMap(List<String> contents) {
        Map<String, OperationDiscussionDoc> retMap = Maps.newHashMap();
        if (ListUtil.isEmpty(contents)) {
            return retMap;
        }
        int index = 1;
        String operationName = null;
        for (String content : contents) {
            if (StringUtil.isBlank(content)) {
                continue;
            }
            operationName = index + "";
            OperationDiscussionDoc operationDiscussionDoc = getOperationDiscussionDoc(content);
            operationDiscussionDoc.setOperationName(operationName);
            retMap.put(operationName, operationDiscussionDoc);
            index++;
        }
        return retMap;
    }

    private OperationDiscussionDoc getOperationDiscussionDoc(String content) {
        Map<String, String> pageData = Preproc.extract_doc_pub(true, operationDiscussion_pageDataTitles, content);
        pageData.put("病程记录内容",
                pageData.get("病程记录内容")
                        .replace(pageData.get("病程记录时间"), "")
                        .replace(pageData.get("病程记录名称"), "")
        );

        List<String> targetTitles = Lists.newArrayList();
        operationDiscussion_sourceTitles.forEach(sourceTitle -> {
            String targetTitle = "";
            for (int index = 0; index < sourceTitle.length(); index++) {
                if (index == sourceTitle.length() - 1) {
                    targetTitle += sourceTitle.substring(index, index + 1);
                } else {
                    targetTitle += sourceTitle.substring(index, index + 1) + "[\\s\\p{Zs}]*";
                }
            }
            targetTitles.add(targetTitle);
        });

        Map<String, String> sourceMap = Preproc.extract_doc_pub(true, targetTitles, content);
        sourceMap.put("记录时间", sourceMap.get("病程记录时间"));

        OperationDiscussionDoc operationDiscussionDoc = ModelDocGenerate.operationDiscussionDocGen(sourceMap);
        operationDiscussionDoc.setText(content);
        operationDiscussionDoc.setPageData((Map) pageData);

        return operationDiscussionDoc;
    }

    private List<String> operationDiscussion_pageDataTitles = Lists.newArrayList(
            "病程记录时间",
            "病程记录名称",
            "病程记录内容",
            "记录医师"
    );
    private List<String> operationDiscussion_sourceTitles = Lists.newArrayList(
            "病程记录时间",
            "病程记录名称",
            "病程记录内容",
            "记录医师",
            "记录时间",
            "手术时间",
            "麻醉方式",
            "手术方式",
            "术中诊断",
            "手术简要经过",
            "患者术后情况",
            "术后处理措施",
            "术后可能出现的并发症及预防措施",
            "术后注意事项",
            "谈话医师签字",
            "患者签名",
            "患者签名时间",
            "被授权人/代理人签名",
            "代理人签名时间"
    );


    /**********************************************手术记录*************************************************************/
    private Map<String, OperationRecordDoc> getOperationRecordDocMap(List<String> contents) {
        Map<String, OperationRecordDoc> retMap = Maps.newHashMap();
        if (ListUtil.isEmpty(contents)) {
            return retMap;
        }
        int index = 1;
        String operationName = null;
        for (String content : contents) {
            if (StringUtil.isBlank(content)) {
                continue;
            }
            operationName = index + "";
            OperationRecordDoc operationRecordDoc = getOperationRecordDoc(content);
            operationRecordDoc.setOperationName(operationName);
            retMap.put(operationName, operationRecordDoc);
            index++;
        }
        return retMap;
    }

    private OperationRecordDoc getOperationRecordDoc(String content) {
        Map<String, String> pageData = Preproc.extract_doc_pub(true, operationRecord_pageDataTitles, content);
        pageData.put("病程记录内容",
                pageData.get("病程记录内容")
                        .replace(pageData.get("病程记录时间"), "")
                        .replace(pageData.get("病程记录名称"), "")
        );

        List<String> targetTitles = Lists.newArrayList();
        operationRecord_sourceTitles.forEach(sourceTitle -> {
            String targetTitle = "";
            for (int index = 0; index < sourceTitle.length(); index++) {
                if (index == sourceTitle.length() - 1) {
                    targetTitle += sourceTitle.substring(index, index + 1);
                } else {
                    targetTitle += sourceTitle.substring(index, index + 1) + "[\\s\\p{Zs}]*";
                }
            }
            targetTitles.add(targetTitle);
        });

        Map<String, String> sourceMap = Preproc.extract_doc_pub(true, targetTitles, content);
        sourceMap.put("记录时间", sourceMap.get("病程记录时间"));

        OperationRecordDoc operationRecordDoc = ModelDocGenerate.operationRecordDocGen(sourceMap);
        operationRecordDoc.setText(content);
        operationRecordDoc.setPageData((Map) pageData);

        return operationRecordDoc;
    }

    private List<String> operationRecord_pageDataTitles = Lists.newArrayList(
            "病程记录时间",
            "病程记录名称",
            "病程记录内容",
            "记录医师"
    );
    private List<String> operationRecord_sourceTitles = Lists.newArrayList(
            "病程记录时间",
            "病程记录名称",
            "病程记录内容",
            "记录医师",
            "记录时间",
            "手术名称",
            "麻醉方式",
            "术前诊断",
            "术后诊断",
            "手术经过"
    );


    /**********************************************术前讨论、术前小结****************************************************/
    private Map<String, PreoperativeDiscussionDoc> getPreoperativeDiscussionDocMap(List<String> contents) {
        Map<String, PreoperativeDiscussionDoc> retMap = Maps.newHashMap();
        if (ListUtil.isEmpty(contents)) {
            return retMap;
        }
        int index = 1;
        String operationName = null;
        for (String content : contents) {
            if (StringUtil.isBlank(content)) {
                continue;
            }
            operationName = index + "";
            PreoperativeDiscussionDoc preoperativeDiscussionDoc = getPreoperativeDiscussionDoc(content);
            preoperativeDiscussionDoc.setOperationName(operationName);
            retMap.put(operationName, preoperativeDiscussionDoc);
            index++;
        }
        return retMap;
    }

    private PreoperativeDiscussionDoc getPreoperativeDiscussionDoc(String content) {
        Map<String, String> xmlNodeValueMap = CxXmlUtil.firstLevelNodeValue("//DocObjContent//Region", content);
        Map<String, String> structureMap = OrdinaryAssistant.mapKeyContrast(xmlNodeValueMap, preoperativeDiscussion_keyContrasts);
        structureMap.put(
                "术前准备内容",
                (StringUtil.isBlank(structureMap.get("术前准备内容")) ? "" : structureMap.get("术前准备内容")) + (StringUtil.isBlank(structureMap.get("术前准备++++术前准备情况")) ? "" : structureMap.get("术前准备++++术前准备情况"))
        );
        structureMap.put(
                "术前术后注意事项",
                (StringUtil.isBlank(structureMap.get("术前术后注意事项")) ? "" : structureMap.get("术前术后注意事项")) + (StringUtil.isBlank(structureMap.get("术后注意事项")) ? "" : structureMap.get("术后注意事项"))
        );
        structureMap.put("讨论时间", structureMap.get("记录时间"));
        if (StringUtil.isNotBlank(structureMap.get("签名"))) {
            structureMap.put("记录医师", structureMap.get("签名"));
        }

        String text = CxXmlUtil.getTextByNodePath(content, "//DocObjContent/Region/Content_Text");
        if (StringUtil.isBlank(text)) {
            text = CxXmlUtil.getTextByNodePath(content, "//DocObjContent/Region");
        }
        Map<String, String> cutWordMap = getCutWordMap(text);
        cutWordMap.putAll(structureMap);

        PreoperativeDiscussionDoc preoperativeDiscussionDoc = ModelDocGenerate.preoperativeDiscussionDocGen(cutWordMap);
        preoperativeDiscussionDoc.setText(text);
        preoperativeDiscussionDoc.setPageData((Map) xmlNodeValueMap);

        return preoperativeDiscussionDoc;
    }

    private Map<String, String> getCutWordMap(String content) {
        if (StringUtil.isBlank(content)) {
            return Maps.newHashMap();
        }
        List<String> targetTitles = Lists.newArrayList();
        preoperativeDiscussion_sourceTitles.forEach(sourceTitle -> {
            String targetTitle = "";
            for (int index = 0; index < sourceTitle.length(); index++) {
                if (index == sourceTitle.length() - 1) {
                    targetTitle += sourceTitle.substring(index, index + 1);
                } else {
                    targetTitle += sourceTitle.substring(index, index + 1) + "[\\s\\p{Zs}]*";
                }
            }
            targetTitles.add(targetTitle);
        });
        if (ListUtil.isEmpty(targetTitles)) {
            return Maps.newHashMap();
        }
        return Preproc.extract_doc_pub(true, targetTitles, content);
    }

    private List<String> preoperativeDiscussion_sourceTitles = Lists.newArrayList(
            "记录医师",
            "记录时间",
            "讨论时间",
            "讨论方式",
            "参加人员",
            "简要病情",
            "讨论结论",
            "经治医师签名",
            "经治医师签字时间",
            "主刀医师签名",
            "主刀医师签字时间",
            "手术指征",
            "麻醉方式",
            "术前准备内容",
            "术前术后注意事项",
            "可能意外和防范措施",
            "拟行术式"
    );

    private List<String> preoperativeDiscussion_keyContrasts = Lists.newArrayList(
            "讨论时间++++记录日期=记录时间",
            "参加人员姓名=参加人员",
            "病情摘要++++简要病情=简要病情",
            "术前诊断++++初步诊断=",
            "手术指征与禁忌征++++手术指征=手术指征",
            "拟行治疗指征及禁忌症=",
            "拟施麻醉方式=麻醉方式",
            "术前准备情况=术前准备内容",
            "术前准备++++术前准备情况=",
            "手术风险及预防措施++++文本框=可能意外和防范措施",
            "术中注意事项=术前术后注意事项",
            "术后注意事项="
    );

}