package com.lantone.qc.trans.changx;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.lantone.qc.pub.model.doc.ThreeLevelWardDoc;
import com.lantone.qc.pub.model.vo.MedrecVo;
import com.lantone.qc.pub.util.StringUtil;
import com.lantone.qc.trans.ModelDocTrans;
import com.lantone.qc.trans.changx.util.CxXmlUtil;
import com.lantone.qc.trans.comsis.ModelDocGenerate;
import com.lantone.qc.trans.comsis.OrdinaryAssistant;
import com.lantone.qc.trans.comsis.Preproc;

import java.util.List;
import java.util.Map;

/**
 * @Description: 查房记录文档生成
 * @author: rengb
 * @time: 2020/3/17 13:27
 */
public class ChangxThreeLevelWardDocTrans extends ModelDocTrans {

    @Override
    public List<ThreeLevelWardDoc> extract(MedrecVo medrecVo) {
        List<ThreeLevelWardDoc> retList = Lists.newArrayList();
        List<String> contents = (List) medrecVo.getContent().get("content");
        contents.forEach(content -> {
            retList.add(getThreeLevelWardDoc(content));
        });
        return retList;
    }

    private ThreeLevelWardDoc getThreeLevelWardDoc(String content) {
        String text = CxXmlUtil.getTextByNodePath(content, "//DocObjContent/Region/Content_Text");
        if (StringUtil.isBlank(text)) {
            text = CxXmlUtil.getTextByNodePath(content, "//DocObjContent/Region");
        }
        Map<String, String> cutWordMap = getCutWordMap(text);
        Map<String, String> xmlNodeValueMap = CxXmlUtil.firstLevelNodeValue("//DocObjContent//Region", content);
        cutWordMap.putAll(xmlNodeValueMap);
        Map<String, String> structureMap = OrdinaryAssistant.mapKeyContrast(cutWordMap, keyContrasts);
        structureMap.put("记录时间", structureMap.get("记录日期"));
        structureMap.put("查房日期", structureMap.get("记录日期"));
        structureMap.put("病情记录", text);
        int endIndex = text.indexOf("查房记录");
        if (endIndex > 0) {
            String title = text.substring(0, endIndex);
            title = title.substring(title.lastIndexOf(" ") + 1) + "查房记录";
            structureMap.put("查房标题", title);
        }
        ThreeLevelWardDoc threeLevelWardDoc = ModelDocGenerate.threeLevelWardDocGen(structureMap);
        threeLevelWardDoc.setText(text);
        threeLevelWardDoc.setPageData((Map) xmlNodeValueMap);
        return threeLevelWardDoc;
    }

    private Map<String, String> getCutWordMap(String content) {
        if (StringUtil.isBlank(content)) {
            return Maps.newHashMap();
        }
        List<String> targetTitles = Lists.newArrayList();
        sourceTitles.forEach(sourceTitle -> {
            String targetTitle = "";
            for (int index = 0; index < sourceTitle.length(); index++) {
                if (index == sourceTitle.length() - 1) {
                    targetTitle += sourceTitle.substring(index, index + 1);
                } else {
                    targetTitle += sourceTitle.substring(index, index + 1) + "[\\s\\p{Zs}]*";
                }
            }
            targetTitles.add(targetTitle);
        });
        return Preproc.extract_doc_pub(true, targetTitles, content);
    }

    private List<String> sourceTitles = Lists.newArrayList(
            "记录医生"
    );

    private List<String> keyContrasts = Lists.newArrayList(
            "记录医生=记录医师"
    );

}